@@ -110,7 +110,7 @@
 	      (if (pair? (cddr val))
 		  `(set! (setter ',(car val))
 			 (if (not (,(caddr val) ,(car val))) ; check initial value (already set)
-			     (error 'wrong-type "initial value ~S is not ~S" ,(car val) ,(caddr val))
+			     (error 'wrong-type "typed-let: initial value ~S is not ~S" ,(car val) ,(caddr val))
 			     ,(caddr val)))                  ; assume built-in type here
 		  (values)))
 	    vals)
@@ -150,9 +150,10 @@
     (if (and (procedure? bp)
 	     (signature bp)
 	     (eq? 'boolean? (car (signature bp))))
-	(if (type e)
-	    e
-	    (error 'bad-type "~S is ~S but should be ~S" e (type-of e) bp))
+	(let ((result (if (= (car (arity bp)) 1)
+			  (type e)
+			  (bp 'the e))))
+	  (if result e (error 'bad-type "~S is ~S but should be ~S" e (type-of e) bp)))
 	(error 'bad-type "~S is not a boolean procedure" bp))))
 
 (define iota
@@ -207,7 +208,6 @@
     (lambda (obj)
       (pair? (cyclic-sequences obj)))))
 
-
 (define copy-tree
   (let ((+documentation+ "(copy-tree lst) returns a full copy of lst"))
     (lambda (lis)
@@ -394,7 +394,7 @@
 	       (set! result (cons `(set! (setter (quote ,(caar var))) (list-ref ,gsetters ,i)) result))))
        ,@body)))
 
-(define-macro (while test . body)      ; while loop with predefined break and continue
+(define-macro (while test . body)      ; while loop with predefined break and continue.  This really wants to be define-expansion.
   `(call-with-exit
     (lambda (break)
       (let loop ()
@@ -979,37 +979,38 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 				 (cons (car slot) (cadr slot))
 				 (cons slot #f)))
 			   ,slots)                    ; the incoming new slots, #f is the default value
-		      new-slots))))                   ; the inherited slots
+		      new-slots)))                    ; the inherited slots
 
-     (set! new-methods
-	   (append (map (lambda (method)
-			  (if (pair? method)
-			      (cons (car method) (cadr method))
-			      (cons method #f)))
-			,methods)                     ; the incoming new methods
-
-		   ;; add an object->string method for this class (this is already a generic function).
-		   (list (cons 'object->string
-			       (lambda (obj . rest)
-				 (if (and (pair? rest)
-					  (eq? (car rest) :readable))    ; write readably
-				     (format #f "(make-~A~{ :~A ~W~^~})"
-					     ',class-name
-					     (map (lambda (slot)
-						    (values (car slot) (cdr slot)))
-						  obj))
-				     (format #f "#<~A: ~{~A~^ ~}>"
-					     ',class-name
-					     (map (lambda (slot)
-						    (list (car slot) (cdr slot)))
-						  obj))))))
-		   (reverse! new-methods)))                      ; the inherited methods, shadowed automatically
+       (set! new-methods
+	     (remove-duplicates
+	      (append (map (lambda (method)
+			     (if (pair? method)
+				 (cons (car method) (cadr method))
+				 (cons method #f)))
+			   ,methods)                     ; the incoming new methods
+		      
+		      ;; add an object->string method for this class (this is already a generic function).
+		      (list (cons 'object->string
+				  (lambda (obj . rest)
+				    (if (and (pair? rest)
+					     (eq? (car rest) :readable))    ; write readably
+					(format #f "(make-~A~{ :~A ~W~^~})"
+						',class-name
+						(map (lambda (slot)
+						       (values (car slot) (cdr slot)))
+						     obj))
+					(format #f "#<~A: ~{~A~^ ~}>"
+						',class-name
+						(map (lambda (slot)
+						       (list (car slot) (cdr slot)))
+						     obj))))))
+		      (reverse! new-methods)))))                  ; the inherited methods, shadowed automatically
 
      (let ((new-class (openlet
                        (apply sublet                             ; the local slots
 			      (sublet                            ; the global slots
 				  (apply inlet                   ; the methods
-					 (reverse new-methods))
+					 new-methods)
 				'class-name ',class-name         ; class-name slot
 				'inherited ,inherited-classes
 				'inheritors ())                  ; classes that inherit from this class
@@ -1447,57 +1448,14 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 
 ;;; --------------------------------------------------------------------------------
 
-(define (*s7*->list) ;(let->list *s7*) but not using keywords
-  (list
-   :print-length                  (*s7* 'print-length)
-   :safety                        (*s7* 'safety)
-   :debug                         (*s7* 'debug)
-   :profile                       (*s7* 'profile)
-   :cpu-time                      (*s7* 'cpu-time)
-   :heap-size                     (*s7* 'heap-size)
-   :free-heap-size                (*s7* 'free-heap-size)
-   :max-heap-size                 (*s7* 'max-heap-size)
-   :max-port-data-size            (*s7* 'max-port-data-size)
-   :gc-freed                      (*s7* 'gc-freed)
-   :gc-total-freed                (*s7* 'gc-total-freed)
-   :gc-protected-objects          (*s7* 'gc-protected-objects)
-   :gc-stats                      (*s7* 'gc-stats)
-   :gc-temps-size                 (*s7* 'gc-temps-size)
-   :gc-resize-heap-fraction       (*s7* 'gc-resize-heap-fraction)
-   :gc-resize-heap-by-4-fraction  (*s7* 'gc-resize-heap-by-4-fraction)
-   :max-string-length             (*s7* 'max-string-length)
-   :max-list-length               (*s7* 'max-list-length)
-   :max-vector-length             (*s7* 'max-vector-length)
-   :max-vector-dimensions         (*s7* 'max-vector-dimensions)
-   :default-hash-table-length     (*s7* 'default-hash-table-length)
-   :initial-string-port-length    (*s7* 'initial-string-port-length)
-   :output-port-data-size         (*s7* 'output-port-data-size)
-   :default-rationalize-error     (*s7* 'default-rationalize-error)
-   :default-random-state          (*s7* 'default-random-state)
-   :equivalent-float-epsilon      (*s7* 'equivalent-float-epsilon)
-   :hash-table-float-epsilon      (*s7* 'hash-table-float-epsilon)
-   :float-format-precision        (*s7* 'float-format-precision)
-   :bignum-precision              (*s7* 'bignum-precision)
-   :file-names                    (*s7* 'file-names)
-   :rootlet-size                  (*s7* 'rootlet-size)
-   :c-types                       (*s7* 'c-types)
-   :stack-top                     (*s7* 'stack-top)
-   :stack-size                    (*s7* 'stack-size)
-   :stacktrace-defaults           (*s7* 'stacktrace-defaults)
-   :max-stack-size                (*s7* 'max-stack-size)
-   :autoloading?                  (*s7* 'autoloading?)
-   :undefined-identifier-warnings (*s7* 'undefined-identifier-warnings)
-   :undefined-constant-warnings   (*s7* 'undefined-constant-warnings)
-   :accept-all-keyword-arguments  (*s7* 'accept-all-keyword-arguments)
-   :catches                       (*s7* 'catches)
-   :history-size                  (*s7* 'history-size)
-   :history-enabled               (*s7* 'history-enabled)
-   :history                       (*s7* 'history)
-   :openlets                      (*s7* 'openlets)
-   :version                       (*s7* 'version)
-   :most-positive-fixnum          (*s7* 'most-positive-fixnum)
-   :most-negative-fixnum          (*s7* 'most-negative-fixnum)))
-
+(define (*s7*->list) ; (let->list *s7*) flattened and using keywords
+  (do ((p (let->list *s7*) (cdr p)) 
+       (L ()))
+      ((null? p) 
+       (reverse L))
+    (set! L (cons (cdar p) 
+		  (cons (symbol->keyword (caar p))
+			L)))))
 
 
 ;;; --------------------------------------------------------------------------------
